/**
 * SPDX-FileCopyrightText: Copyright (c) 2018-2021, NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#pragma once

namespace nvrpc {
namespace client {

class BaseContext
{
  public:
    BaseContext() : m_MasterContext(this) {}
    BaseContext(BaseContext* master) : m_MasterContext(master) {}

    virtual ~BaseContext() {}
    virtual bool RunNextState(bool ok) = 0;

    virtual bool ExecutorShouldDeleteContext() const = 0;

    void* Tag()
    {
        return static_cast<void*>(this);
    }

    static BaseContext* Detag(void* tag)
    {
        return static_cast<BaseContext*>(tag);
    }

  protected:
    BaseContext* m_MasterContext;
};

}  // namespace client
}  // namespace nvrpc
